@@ -27,6 +27,7 @@
 #' When \code{hansen} is executed, the numerical optimizer maximizes the likelihood over these parameters.
 #'
 #' @name hansen
+#' @aliases hansentree-class
 #' @rdname hansen
 #' @author Aaron A. King
 #' @seealso
@@ -59,7 +60,17 @@ setClass(
   )
 )
 
-NULL
+setAs(
+  from='hansentree',
+  to='data.frame',
+  def = function (from) {
+    cbind(
+      as(as(from,'ouchtree'),'data.frame'),
+      as.data.frame(from@regimes),
+      as.data.frame(from@data)
+    )
+  }
+)
 
 #' @rdname hansen
 #' @include ouchtree.R glssoln.R rmvnorm.R
@@ -82,7 +93,7 @@ NULL
 #' The selection strength matrix \eqn{\alpha}{alpha} and the random drift variance-covariance matrix \eqn{\sigma^2}{sigma^2} are parameterized by their matrix square roots.
 #' Specifically, these initial guesses are each packed into lower-triangular matrices (column by column).
 #' The product of this matrix with its transpose is the \eqn{\alpha}{alpha} or \eqn{\sigma^2}{sigma^2} matrix.
-#' See Details.
+#' See Details for more information.
 #' @param fit If \code{fit=TRUE}, then the likelihood will be maximized.
 #' If \code{fit=FALSE}, the likelihood will be evaluated at the specified values of \code{sqrt.alpha} and \code{sigma};
 #' the optima \code{theta} will be returned as well.
@@ -93,9 +104,7 @@ NULL
 #' See \code{\link{optim}} and \code{\link[subplex:subplex]{subplex}} for information on the available options.
 #' 
 #' @return \code{hansen} returns an object of class \code{hansentree}.
-#' For details on the methods of that class, see \code{\link{hansen}}.
-#' 
-#' @export hansen
+#' @export
 hansen <- function (data, tree, regimes, sqrt.alpha, sigma,
   fit = TRUE,
   method = c("Nelder-Mead","subplex","BFGS","L-BFGS-B"),
@@ -132,7 +141,8 @@ hansen <- function (data, tree, regimes, sqrt.alpha, sigma,
     for (xx in data) {
       no.dats <- which(is.na(xx[tree@nodes[tree@term]]))
       if (length(no.dats)>0)
-        stop("missing data on terminal node(s): ",paste(tree@nodes[tree@term[no.dats]],collapse=','))
+        stop("missing data on terminal node(s): ",
+          paste(sQuote(tree@nodes[tree@term[no.dats]]),collapse=', '))
     }
   } else
     stop(sQuote("data")," must be either a single numeric data set or a list of numeric data sets")
@@ -331,17 +341,17 @@ ou.lik.fn <- function (tree, alpha, sigma, beta, dat) {
 sym.par <- function (x) {
   nchar <- floor(sqrt(2*length(x)))
   if (nchar*(nchar+1)!=2*length(x)) {
-    stop("a symmetric matrix is parameterized by a triangular number of parameters",call.=FALSE)
+    stop("a symmetric matrix is parameterized by a triangular number of parameters",call.=FALSE) #nocov
   }
   y <- matrix(0,nchar,nchar)
   y[lower.tri(y,diag=TRUE)] <- x
   y%*%t(y)
 }
 
-sym.unpar <- function (x) {
-  y <- t(chol(x))
-  y[lower.tri(y,diag=TRUE)]
-}
+## sym.unpar <- function (x) {
+##   y <- t(chol(x))
+##   y[lower.tri(y,diag=TRUE)]
+## }
 
 sets.of.regimes <- function (object, regimes) {
   lapply(regimes,function(x)sort(unique(x)))
@@ -367,22 +377,23 @@ regime.spec <- function (object, regimes) {
   beta
 }
 
-## solve the matrix equation
+## Solve the matrix equation
 ##   A . X + X . A = B
 ## for X, where we have assumed A = A'.
-sym.solve <- function (a, b) {
-  n <- nrow(a)
-  d <- array(data=0,dim=c(n,n,n,n))
-  for (k in seq_len(n)) {
-    d[k,,k,] <- d[k,,k,] + a
-    d[,k,,k] <- d[,k,,k] + a
-  }
-  dim(b) <- n*n
-  dim(d) <- c(n*n,n*n)
-  x <- solve(d,b)
-  dim(x) <- c(n,n)
-  x
-}
+## 
+## sym.solve <- function (a, b) {
+##   n <- nrow(a)
+##   d <- array(data=0,dim=c(n,n,n,n))
+##   for (k in seq_len(n)) {
+##     d[k,,k,] <- d[k,,k,] + a
+##     d[,k,,k] <- d[,k,,k] + a
+##   }
+##   dim(b) <- n*n
+##   dim(d) <- c(n*n,n*n)
+##   x <- solve(d,b)
+##   dim(x) <- c(n,n)
+##   x
+## }
 
 hansen.deviate <- function (n = 1, object) {
   ev <- eigen(sym.par(object@sqrt.alpha),symmetric=TRUE)
@@ -411,49 +422,9 @@ hansen.deviate <- function (n = 1, object) {
   apply(X,3,as.data.frame)
 }
 
-#' @rdname simulate
-#' @include simulate.R
-#' @importFrom stats runif
-#' @export
-setMethod(
-  'simulate',
-  signature=signature(object='hansentree'),
-  function (object, nsim = 1, seed = NULL, ...) {
-    if (!is.null(seed)) {
-      if (!exists('.Random.seed',envir=.GlobalEnv)) runif(1)
-      save.seed <- get('.Random.seed',envir=.GlobalEnv)
-      set.seed(seed)
-    }
-    X <- hansen.deviate(n=nsim,object)
-    if (!is.null(seed)) {
-      assign('.Random.seed',save.seed,envir=.GlobalEnv)
-    }
-    X
-  }
-)
-
-#' @rdname update
-#' @importFrom stats update
-#' @inheritParams hansen
-#' @export
-setMethod(
-  'update',
-  signature=signature(object='hansentree'),
-  function (object, data, regimes, sqrt.alpha, sigma, ...) {
-    if (missing(sqrt.alpha)) sqrt.alpha <- object@sqrt.alpha
-    if (missing(sigma)) sigma <- object@sigma
-    hansen(
-      data=data,
-      tree=object,
-      regimes=regimes,
-      sqrt.alpha=sqrt.alpha,
-      sigma=sigma,
-      ...
-    )
-  }
-)
-
 #' @rdname coef
+#' @aliases coef-hansentree, coef,hansentree-method
+#' @include coef.R
 #' @importFrom stats coef
 #' @return \code{coef} applied to a \code{hansentree} object returns a named list containing the estimated \eqn{\alpha}{alpha} and \eqn{\sigma^2}{sigma^2} matrices(given as the \code{alpha.matrix} and \code{sigma.sq.matrix} elements, respectively) but also the MLE returned by the optimizer
 #' (as \code{sqrt.alpha} and \code{sigma}, respectively).
@@ -473,7 +444,9 @@ setMethod(
   }
 )
 
-#' @rdname loglik
+#' @rdname logLik
+#' @aliases logLik-hansentree, logLik,hansentree-method
+#' @include logLik.R
 #' @importFrom stats logLik
 #' @export
 setMethod(
@@ -483,6 +456,8 @@ setMethod(
 )
 
 #' @rdname summary
+#' @aliases summary-hansentree, summary,hansentree-method
+#' @include summary.R
 #' @return \code{summary} applied to a \code{hansentree} method displays the estimated \eqn{\alpha}{alpha} and \eqn{\sigma^2}{sigma^2} matrices as well as various quantities describing the goodness of model fit.
 #' @export
 setMethod(
@@ -515,6 +490,8 @@ setMethod(
 )
 
 #' @rdname print
+#' @aliases print-hansentree, print,hansentree-method
+#' @include print.R
 #' @return \code{print} displays the tree as a table, along with the coefficients of the fitted model and diagnostic information.
 #' @export
 setMethod(
@@ -543,6 +520,8 @@ setMethod(
 )
 
 #' @rdname print
+#' @aliases show-hansentree, show,hansentree-method
+#' @include print.R
 #' @export
 setMethod(
   'show',
@@ -553,14 +532,73 @@ setMethod(
   }
 )
 
-setAs(
-  from='hansentree',
-  to='data.frame',
-  def = function (from) {
-    cbind(
-      as(as(from,'ouchtree'),'data.frame'),
-      as.data.frame(from@regimes),
-      as.data.frame(from@data)
+#' @rdname plot
+#' @aliases plot,hansentree-method plot-hansentree
+#' @include plot.R
+#' @export
+setMethod(
+  "plot",
+  signature=signature(x="hansentree"),
+  function (x, ..., regimes) {
+    if (missing(regimes)) regimes <- x@regimes
+    f <- getMethod("plot","ouchtree")
+    f(x=x,regimes=regimes,...)
+  }
+)
+
+#' @rdname simulate
+#' @aliases simulate-hansentree, simulate,hansentree-method
+#' @include simulate.R package.R
+#' @importFrom stats runif
+#' @export
+setMethod(
+  'simulate',
+  signature=signature(object='hansentree'),
+  function (object, nsim = 1, seed = NULL, ...) {
+    seed <- freeze(seed)
+    X <- hansen.deviate(n=nsim,object)
+    thaw(seed)
+    X
+  }
+)
+
+#' @rdname update
+#' @aliases update-hansentree, update,hansentree-method
+#' @include update.R
+#' @importFrom stats update
+#' @inheritParams hansen
+#' @export
+setMethod(
+  'update',
+  signature=signature(object='hansentree'),
+  function (object, data, regimes, sqrt.alpha, sigma, ...) {
+    if (missing(sqrt.alpha)) sqrt.alpha <- object@sqrt.alpha
+    if (missing(sigma)) sigma <- object@sigma
+    hansen(
+      data=data,
+      tree=object,
+      regimes=regimes,
+      sqrt.alpha=sqrt.alpha,
+      sigma=sigma,
+      ...
     )
   }
 )
+
+#' @rdname bootstrap
+#' @aliases bootstrap-hansentree, bootstrap,hansentree-method
+#' @include bootstrap.R
+#' @export
+setMethod(
+  "bootstrap",
+  signature=signature(object="hansentree"),
+  function (object, nboot = 200, seed = NULL, ...) {
+    simdata <- simulate(object,nsim=nboot,seed=seed)
+    results <- vector(mode='list',length=nboot)
+    toshow <- c("alpha","sigma.squared","optima","loglik","aic","aic.c","sic","dof")
+    for (b in seq_len(nboot)) {
+      results[[b]] <- summary(update(object,data=simdata[[b]],...))
+    }
+    as.data.frame(t(sapply(results,function(x)unlist(x[toshow]))))
+  }
+)
