@@ -60,7 +60,7 @@ ouchtree <- function (nodes, ancestors, times, labels = as.character(nodes)) {
   ancestors <- as.character(ancestors)
 
   n <- length(nodes)
-  if (length(unique(nodes)) != n) stop("node names must be unique")
+  if (anyDuplicated(nodes)>0) stop("node names must be unique")
   if (length(ancestors) != n)
     stop("invalid tree: ",sQuote("ancestors")," must have the same length as ",sQuote("nodes"))
   if (length(times) != n) 
@@ -75,20 +75,24 @@ ouchtree <- function (nodes, ancestors, times, labels = as.character(nodes)) {
     stop("the algorithms assume that the root node is at time=0")
   
   term <- terminal.twigs(nodes,ancestors)
-  if (length(term) <= 0) 
-    stop("invalid tree: there ought to be at least one terminal node, don't you think?")
+  if (length(term) <= 0)
+    stop("invalid tree: there ought to be at least one terminal node, don't you think?") #nocov
 
   outs <- which((!is.root.node(ancestors) & !(ancestors %in% nodes)))
   if (length(outs) > 0) {
-    for (out in outs) {
-      warning(
-        sprintf(
-          "the ancestor of node %s is not in the tree",
-          nodes[out]
-        ),
-        call.=FALSE)
-    }
-    stop("invalid tree")
+    stop(
+      ngettext(
+        length(outs),
+        "invalid tree: the ancestor of node ",
+        "invalid tree: the ancestors of nodes "
+      ),
+      paste(sQuote(nodes[outs]),collapse=", "),
+      ngettext(
+        length(outs),
+        " is not in the tree.",
+        " are not in the tree."
+      )
+    )
   }
   
   anc <- ancestor.numbers(nodes,ancestors)
@@ -98,7 +102,6 @@ ouchtree <- function (nodes, ancestors, times, labels = as.character(nodes)) {
     stop("this is no tree: node ",nodes[w[1]]," is its own ancestor",call.=FALSE)
   }
 
-###  lineages <- build.lineages(anc)
   lineages <- vector(mode='list',length=n)
   todo <- root
   k <- 1
@@ -141,29 +144,7 @@ ancestor.numbers <- function (nodenames, ancestors) {
 
 ## nodenames of terminal twigs (terminal nodes are not ancestors)
 terminal.twigs <- function (nodenames, ancestors) {
-  which(nodenames %in% setdiff(nodenames,unique(ancestors)))
-}
-
-build.lineages <- function (ancestors) {
-  n <- length(ancestors)
-  lineages <- vector(mode='list',length=n)
-  pedigree <- function (k) {
-    if (is.null(lineages[[k]])) {
-      a <- ancestors[k]
-      if (is.root.node(a)) {
-        lineages[[k]] <<- k
-      } else {
-        if (is.null(lineages[[a]])) Recall(a)
-        if (k %in% lineages[[a]]) 
-          stop('this is no tree: circularity detected at node ',k,call.=FALSE)
-        lineages[[k]] <<- c(k,lineages[[a]])
-      }
-    }
-    NULL
-  }
-  for (k in 1:n)
-    pedigree(k)
-  lineages
+  which(nodenames %in% setdiff(nodenames,ancestors))
 }
 
 branch.times <- function (lineages, times, term) {
@@ -198,6 +179,7 @@ is.root.node <- function (anc) {
 }
 
 #' @rdname print
+#' @include print.R
 #' @export
 setMethod(
   'print',
@@ -209,6 +191,7 @@ setMethod(
 )
 
 #' @rdname print
+#' @include print.R
 #' @export
 setMethod(
   'show',
